#!/usr/bin/perl

#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
#	This script currently supports the following videosites:
#		[*] youtube
#		[*] myvideo
#		[*] clipfish
#		[*] metacafe
#   changelog:	
#		2007-10-20	initial release
#		2007-10-20	added avi/mp3-support (encoding)
#		2007-10-20	improved code for command switches	
#				added renaming-support
#		2007-10-28	added support for myvideo & stage6
#		2007-10-29	added support for changing the videopath 
#				fixed some bugs
#		2007-11-03	added support for just printing the downloadlinks
#				realized that the code is really ugly...
#					=> TODO: rewrite the whole thing,	
#						 maybe tomorrow....
#		2007-11-08	added support for de.youtube.com
#		2008-01-30	minor changes to the code, due to youtubes 
#				latest changes....
#		2008-01-31	replaced the old commadlineparser
#		     		with Getopt::Long
#		2008-02-01	Replaced systemcalls of wget/curl with
#				LWP::UserAgent methods
#		2008-04-20	added support for clipfish, though it's
#				still a bit buggy, so don't expect it to work...
#		2008-04-21	renamed the project to
#					*yaydl - yet another youtube downloader*
#				added support for metacafe
#				removed support fpr stage6...R.I.P.
#				miscellaneous changes	 
#		2008-05-02	fixed some bugs
#		2008-06-04	fixed some minor bugs
#				changed code for myvideo due to their latest changes
#				
#
#		If you receive an error like
#		"user@host ~ $ Can't locate LWP/UserAgent.pm in @INC (@INC contains:)..."
#		you have to install the perl module LWP/Useragent
#		On debian-based systems, simply do a  "apt-get install libwww-perl" 
#		Alternative:  use "perl -MCPAN -e shell"...
#		An older version of this script, which still uses curl/wget is also available:
#		http://home.arcor.de/haui.45/scripts/perl/yaydl_youtubedownloader/old/
#

use strict;
use warnings;
#use utf8;
use Getopt::Long;
use LWP::UserAgent;
use MP3::Info;
Getopt::Long::Configure ("bundling", "gnu_compat");

$SIG{INT} = \&catcher;		#Catch Ctrl-C and exit

my %INFO = (
	authors => "Haui",
	contact => "haui45\@web.de",
	date	=> "2008-06-04",
	version => "v0.7.6a",
	license => "GPL",
	name	=> "yaydl - yet another youtube downloader",
);

#*************************************************************************
my $videopath = "/my/videopath/";	#IMPORTANT - change to your path!*
#*************************************************************************

my $vidid = "";		#the "url-id"
my $id = "";		#the "magic-id"
my $path = "";		
my $encode = 0;		#output file for the encoding proces
my $source = "";	#input file for the encoding process
my $name = "";		#new filename
my $ua;			#LWP user agent
my %options = (
		rename	=> '0',
		help	=> '0',
		sound	=> '0',
		version	=> '0',
		encode	=> '0',
		print	=> '0',
		tag	=> '0',
);

sub catcher() {
	die "SIGINT recieved....exiting\n";
}
sub checkargs(){
	if ($#ARGV +1 == 0){
		print "Please specify at least one link...\n";
		exit;
	}
	GetOptions ("rename|r" => \$options{'rename'}, "help|h" => \$options{'help'},
		 "sound|s" => \$options{'sound'}, "version|v" => \$options{'version'}, 
			"encode|e" => \$options{'encode'} , "print" => \$options{'print'},
				"tag|t"=>\$options{'tag'},  "path|p=s" => \$videopath);

	if ($options{'version'} == 1){
		version();
		$options{'version'} = 0;
		exit 0;
	}
	if ($options{'help'} == 1){
		help();
		$options{'help'} = 0;
		exit 0;
	}
	if ($#ARGV +1 == 0){
		print "No URL specified\n";
		exit;
	}
	if ($videopath =~ m/\/my\/videopath\// or $videopath eq ""){
		print "Please set a proper videopath!\n";
		exit;
	}
	#print "@{[ %options ]}\n";
}



sub choose(){
	my ($path, $url,$dec);
	print "\nImportant notice: All links must be quoted correctly -> \"http://www.youtube.com...\"\n";
	print "Type \"y\" to proceed\n";
	print "[y/n]\n";
	$dec = <STDIN>;
	unless ($dec =~ m/y(es)*/i){
		exit 1;
	}
	$ua = LWP::UserAgent->new();
	$ua->agent('Mozilla/5.0');
	foreach (@ARGV){
		print "---\n";
		if ($_ =~ m/^http:\/\/(www\.)*youtube\.com\/watch/ || $_ =~ m/http:\/\/de\.youtube\.com\/watch/){ 
			($url, $path) = youtube($_);
		}
		elsif (($_ =~ m/^http:\/\/www\.myvideo\.de\/watch/ || $_ =~ m/^http:\/\/myvideo\.de\/watch/)){
			($url, $path) = myvideo($_);
		}
		elsif (($_ =~ m#^http://www\.clipfish\.de/player\.php\?videoid=.*#)){
			($url, $path) = clipfish($_);
		}
		elsif (($_ =~ m#^http://www.metacafe.com/watch/.*#)){
			($url, $path) = metacafe($_);
		}
		else{
			print "$_: Not a valid Link!\n";
			next;
		}
		if ($url eq ""){
			next;
		}
		#just print the downloadlink...
		if($options{'print'}){
			print "Downloadlink: $url\n";
			next;
		}
			
		print "Download in progress - this may take a while\n";
		if (download($url, $path) != 0){
			die "Error while getting the file!\n\n";
		}
		print "\n[***************]Saved file $path [***************]\n\n";

		if($options{'encode'} == 1){
			if (encode("$path") == -1){

			}
			else{
			print "\n[**************] Encoded file $encode [****************]\n\n" unless 
				($options{'rename'} == 1);
			}
		}
		if($options{'sound'} == 1){
			if (sound("$path") == -1){
				die "ripping error!\n";
			}
			print "\n[**************] Encoded file $encode [****************]\n\n" unless 
				($options{'rename'} == 1);
		}
		if ($options{'sound'} == 1 || $options{'encode'} == 1 ){
			unlink("$path");
		}
		else{
			if ($options{'rename'} == 1){
				print "\nEnter a new filename\n";
				$name = <STDIN>;
				chomp($name);
				$name = "$videopath" . "/$name";
				rename("$path", "$name") or
					die "renaming failed!\n\n";
				print "\n$path renamed successfully to $name\n\n";
		}
		$path="";
		}

	}
		print "---\n";
}

sub youtube(){
	my ($path, $url) = "";
	my $line = "http://youtube.com/get_video?video_id=" ;
	$vidid = shift;
	$vidid =~ s#http://((de\.)|(www\.))*youtube.com/watch\?v=##;
	$vidid =~ s#\&.*##;
	$path = "$videopath" . "/$vidid" . ".flv";
	
	if ((getpage($_)) != 0){
		print "Error while getting the file!\n\n";
		return "";
	}
	my @testing = split(/\n/, $id);
	my @testme = grep (/^.*, "t": ".*", "hl":.*/, @testing);
	unless(defined $testme[0]){
		print "Error while getting the file!\n\n";
		return "";
	}
	$id = $testme[0];
	$id =~ s/^.*, "t": "//;		#alles vor und inkl. ', "t": "' loeschen
	$id =~ s/", "hl":.*//;		#alles nach und inkl. '", "hl":' loeschen
	$url = $line . $vidid . "\&t=" . $id;	# url zusammenfuegen
	return $url, $path;
}

sub clipfish(){
	my ($url, $serverid) ="";
	$url = shift;
	my $server="http://pg1.clipfish.de/media/";
	if ((getpage($url)) != 0){
		print "Error while getting the file!\n\n";
		return "";
	}
	
	#get the id for the flv file	
	my @array = split(/\n/, $id);
	my @ids = grep (/[ ]*<!--.*flv.*/, @array);
	unless (defined $ids[0]){
		print "Error while getting the file!\n\n";
		return "";
	}
	my @ismusic = grep (/media\/musicvideos/, @array);

	$vidid = $ids[0];
	chomp($vidid);
	$vidid =~ s/.*<!--//;
	$vidid =~ s/\.flv.*/\.flv/;
	#set the download path
	$path = $videopath . $vidid;
	
	#music videos need a special treatment...
	if (defined $ismusic[0]) {
		print "It's a  musicvideo! :D\n ";
		$url = $server . "musicvideos/" . substr($vidid, 0, 2) . 
		"/" . substr($vidid,2,3) . "/" .  substr($vidid, 5,2 ) . "/" . $vidid;

	}
	else {
		#get the two-digit id for the download server	
		$serverid = $vidid;
		$serverid =~ s/\.flv//;
		$serverid = substr($serverid, length($serverid)-2, 2);
	
		#create the url....
		$url = $server . $serverid . "/" . $vidid;
	}	
	return $url, $path;
}
sub metacafe(){
	my $url = shift;
	my $path = $url;
	if ((getpage($url)) != 0){
		print "Error while getting the file!\n\n";
		return "";
	}
	my @array = split("\n", $id);
	my @ids = grep (/so\.addParam\("flashvars", "itemID=/, @array);
	unless (defined $ids[0]){
		print "Error while getting the file!\n\n";
		return "";
	}
	$url = $ids[0];
	$url =~ s#.*mediaURL=##;
	$url =~ s#\.flv.*#\.flv#;
	
	$path =~ s#http://www.metacafe.com/watch/##;
	$path =~ s#/.*##;
	$path = $videopath . $path . ".flv";
	return $url, $path;
}

sub myvideo(){
	my ($path, $url);
	my @swfargs;
	$vidid = shift;

	if ((getpage($vidid)) != 0){
		print "Error while getting the file!\n\n";
		return "";
	}
		
	my @array = split(/\n/, $id);

	$vidid =~ s#(http://)?(www\.)?myvideo\.de/watch/##;
	$vidid =~ s#/.*##;
	$path = "$videopath" . "/$vidid" . ".flv";

	@swfargs = grep /SWFObject\(/, @array;
	unless (defined($swfargs[0])) {
		print "variable swfargs undefined...stopping method\n\n"; 
		return "";
	}
	$id=$swfargs[0];
	undef (@array);
	undef (@swfargs);

	$id =~ s#.*SWFObject\('##;
	$id =~ s#;.*##;
	$id =~ s#.*\?##;
	$id =~ s#\&.*##;
	chomp($id);
	$url = $id;
	return $url, $path;

}

sub download() {
	my ($url, $path) = @_;
	print "Downloalink: $url\n\n";
	my ($request, $response);
	$request = HTTP::Request->new('HEAD', $url);			#	Filesize
	$response = $ua->request($request);				# 	.
	if ($response->is_success()){					#	.
		if (int($response->header('Content-Length')) == 0){
			print "Filesize 0 byte....stopping method!\n\n";
			return -1;
		}
		print "Filesize: " . int($response->header('Content-Length')/1024) . "kb\n"; #
	}								#	.
	else{								#	.
		print "Error code    : ", $response->code() ,    "\n";	#	.
		print "Error message:  ", $response->message() , "\n";	#	.
		return -1;						#	.
	}								#	.
	$request = HTTP::Request->new('GET', $url);			#
	$response = $ua->request($request, $path);			#getting the file...
	if ( $response->is_error() ) { 					#	
	       print "Error code    : ", $response->code() ,    "\n";	#
		print "Error message:  ", $response->message() , "\n";	#
		return 1;						#
	}	
	return 0;
	
}

sub getpage() {
	my $link = shift;

	my ($path, $request, $response, $url) = "";
	$request = HTTP::Request->new('GET', $link);
	$response = $ua->request($request);
	print "Retrieving http-page...\n";
	if ( $response->is_error() ) { 
		print "Error code    : ", $response->code() ,    "\n";
		print "Error message:  ", $response->message() , "\n";
		return -1;
	}
	else {
		$id = $response->content();
		return 0;
	}

}

sub encode(){
	$source = shift;
	$encode = $source;
	$encode =~ s/\.flv/\.avi/;
	$encode =~ s/\.divx/\.avi/;
	my $ffmpeg = `which ffmpeg`;
	my $mencoder = `which mencoder`;
	my @commands;
	my $return;
	if ($mencoder ne ""){
		 push(@commands, "mencoder \"$source\" -oac mp3lame -lameopts abr:br=92" .
		" -ovc xvid -xvidencopts bitrate=150 -o \"$encode\"");
		print "mencoder: found\n";
	}
	if($ffmpeg ne ""){
		push (@commands, "ffmpeg -i \"$source\" \"$encode\"");
		print "ffmpeg: found\n";
	}
	if($ffmpeg eq "" && $mencoder eq ""){
		$options{'encode'} = 0;
		print "mencoder: not found!\nffmpeg: not found!\nencoding disabled\n";
		return -1;
	}
	foreach my $command(@commands){
		$return = system($command);
		if ($return == 0){
			print "success\n";
			last;
		}
	}
	if ($return != 0){
		print "error - encoding disabled\n";
		$options{'encode'} = 0;
		return -1;
	}


	if ($options{'rename'} == 1){
		print "\nEnter a new filename\n";
		$name = <STDIN>;
		chomp($name);
		$name =~ s/.avi$//;
		$name = "$videopath" . "/$name" . ".avi";
		rename("$encode", "$name") or die "encode rename: Something went wrong. Fix it! :)\n";
		print "\n$encode renamed successfully to $name\n\n";
	}

}

sub sound(){
	$source = shift;
	$encode = $source;
	$encode =~ s/\.flv/\.mp3/;
	$encode =~ s/\.divx$/\.mp3/;
	system("mplayer -dumpaudio -dumpfile \"$encode\" \"$source\"") == 0
			or return -1;
	if ($options{'rename'} == 1){
		print "\nEnter a new filename\n";
		$name = <STDIN>;
		chomp($name);
		$name =~ s/.mp3$//;
		$name = "$videopath" . "/$name" . ".mp3";
		rename("$encode", "$name") or die "sound rename: Something went wrong. Fix it!\n";
		print "\n$encode renamed successfully to $name\n\n";
	}
	if ($options{'tag'} == 1){
		eval {
			print "Enter new title: ";
			my $title = <STDIN>;
			chomp($title);
			print "Enter new artist: ";
			my $artist = <STDIN>;
			chomp($artist);
			print "Enter new album: ";
			my $album = <STDIN>;
			chomp($album);
			my $mp3 = new MP3::Info "$name";
			$mp3->title($title);
			$mp3->artist($artist);
			$mp3->album($album);
			print "tagging successful\n\n";
		};
		if($@){
			print "tagging error\n";
			return -1;
		}

	}
	return 0;
}



sub help(){
	print "Aufruf: yaydl.pl \"link1\" \"link2\" ...\n".

	 "\nImportant notice: all links must be quoted correctly!\n".
	 "Needs LWP::Useragent and mencoder/mplayer \n".
	 "Parameters::\n".
	 "   -e, --encode \t enkodiert alle Videos ins divx-Format. Bentigt mencoder!\n".
	 "    \t\t\t yaydl -e \"http://youtube...\"\n".
	 "   -s, --sound\t\t extrahiere nur die Sounddaten. Bentigt mplayer!\n".
	 "    \t\t\t yaydl -s \"http://youtube...\"\n".
	 "   -r, --rename \t benennt alle Dateien um\n" .
	 "   -t, --tag\t\t biete die Mglichkeit Titel/interpret/Album als MP-Tag hinzuzufgen\n" .
	 "   -p, --path PFAD \t ndert den Zielordner auf PFAD\n" . 
	 "       --print \t\t nur Downloadlinks anzeigen\n" . 
	 "   -h, --help   \t Zeig diese Hilfe an\n".
	 "   -v, --version \t Print version.\n";
	
	return;
}
sub version(){
	print "yaydl - yet another youtube downloader \nversion $INFO{'version'} - $INFO{'date'}\n";
	print "Author: $INFO{'authors'}, contact $INFO{'contact'}\n";
	return;
}

checkargs();
choose();

